/************************************************************************/
/*	Copyright (C) 2004  Michael C. Shultz				*/
/*									*/
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or (at*/
/* your option) any later version.					*/
/*									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	*/
/* GNU General Public License for more details. 			*/
/*									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*/
/*  02111-1307, USA.							*/
/* 									*/
/* Michael C. Shultz							*/
/* ringworm@inbox.lv							*/
/* Box 3238 Landers, CA 92285						*/
/************************************************************************/

/************************************************/
/* 	macro index:				*/
/*		MGmSetString( var, string )	*/
/*		MGmDbArray( b, c , d )		*/
/*		MGsdbName( a ) 		*/
/************************************************/
#ifndef	__LIBMG_H__
#define __LIBMG_H__	1

#ifndef __errno__
extern	int	errno;
#define __errno__	1
#endif

/*
#include	<config.h>
*/

#include	<dirent.h>
#include	<signal.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<sys/time.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<sysexits.h>
#include	<unistd.h>

#include	<sys/resource.h>

#ifndef CR
#define	CR	13
#endif

#ifndef LF
#define	LF	10
#endif

#ifndef SPACE
#define	SPACE	32
#endif

#ifndef TAB
#define	TAB	9
#endif

/*.................................................
structure	MGsDb
Used to contain the various elements of a data base

name		= path + name of data base file
mode	 	= mode data base is to be opened
		"r" read only
		"r+" read/write
		"a" append
..................................................*/
typedef struct
{
	char*		buffer; 	/* buffer used to contain data base */
	char*		field;
	char*		mode;		/* mode in which to open file */
	char*		name;		/* name and path of data base file */
	char**		record;
	char**		recordStartAddress;
	char*** 	array;		/* data base array */
	FILE*		stream; 	/* assigned to data base file after it is opened */
	int		bufferPtr1;
	signed	int	eof;		/* size of data base file in bytes */
	int		fieldPtr1;
	int		fieldIdx;
	signed	int	fieldQty;	/* quantity of fields per record */
	int		recordIdx;
	int		recordQty;	/* quantity of records per data base */
} MGsDb;

/*.................................................
					<a name="MGsProperty">
structure	MGsProperty
					</a>

intended use is to provide a consistant means of organizing an
application's top level property structure.  See MGsProperty(7)
for a detailed description 

id		= assigned application's name
version 	= assigned application's version, in a string
feedbNameack	= determines the level of verbosity as application executes
		  best to set this with MGrCommandLine(3) but it may of course
		  be set or adjused elsewhere
..................................................*/
typedef struct
{
	char*	id;
	char*	version;
	int	feedbNameack;
} MGsProperty;


/***********************************
MACROS
************************************/
/***********************************
MGmDbArray:
b = dbNamease  structure name
c = dbNamease file name string
d = open mode string
************************************/
#define MGmDbArrayFree( dbName )	\
	free( dbName.buffer );	\
	free( dbName.mode );	\
	free( dbName.name );	\
	free( dbName.recordStartAddress );	\
	free( dbName.array );	\
	fclose( dbName.stream );
	
#define MGmDbArray( dbName, dbFileName, FileOpenMode )	\
	do{	\
	errno = 0;	\
	if( !( dbName.name = ( char*)malloc( strlen( dbFileName ) + 1 ) ) )	\
	{																												\
		fprintf( stderr, "MGmDbArray error: Unable to allocate memory for %s\n", dbFileName );	\
		perror( "system message" );	\
		errno = 1;	\
	}																												\
	if( errno==0 )	\
	{	\
		strcpy( dbName.name, dbFileName );	\
		if( !( dbName.mode = ( char*)malloc( strlen( FileOpenMode ) + 1 ) ) )	\
		{																											\
			fprintf( stderr, "MGmDbArray error:  Unable to allocate 2 bytes %s mode  %s\n", dbFileName, FileOpenMode );	\
			perror( "system message" );	\
			errno = 1;	\
		}	\
		if( errno==0 )	\
		{	\
			strcpy( dbName.mode, FileOpenMode );\
			if( !( dbName.eof = MGrFileSize( dbName.name ) ) )	\
			{	\
				fprintf( stderr, "MGmDbArray warning:  MGrFileSize returned a size of %d for %s\n", dbName.eof, dbName.name );	\
				errno = 2;	\
			}	\
			if( errno==0 )	\
			{	\
				if(!(dbName.buffer=(char*)malloc((unsigned int)dbName.eof)))	\
				{																									\
					fprintf( stderr, "MGmDbArray error:  unable to	reallocate %d bytes memory for %s\n",	\
										dbName.eof, dbName.name );	\
					perror( "system message" );	\
					errno = 1;	\
				}	\
				if( errno==0 )	\
				{	\
					if( !( dbName.stream = fopen( dbName.name, dbName.mode ) ) )	\
					{	\
						fprintf( stderr, "MGmDbArray error: unable to open file %s\n", dbName.name );	\
						perror( "system message" );	\
						errno = 1;	\
					}	\
					if( errno==0 )	\
					{	\
						fread( dbName.buffer, dbName.eof, 1, dbName.stream );	\
						if( ferror( dbName.stream ) )	\
						{	\
							fprintf( stderr, "MGmDbArray error: reading file %s into dbName.buffer\n", dbName.name );	\
							perror( "system message" );	\
							errno = 1;	\
						}	\
						if( errno==0 )	\
						{	\
							dbName.bufferPtr1	= 0;	\
							dbName.fieldQty	= 0;	\
							while( dbName.buffer[dbName.bufferPtr1] != LF )	\
							{	\
								if( !dbName.buffer[dbName.bufferPtr1] )	\
								{	\
									dbName.fieldQty++;	\
								}	\
								dbName.bufferPtr1++;	\
							}	\
							dbName.bufferPtr1	= 0;	\
							dbName.fieldPtr1	= 0;	\
							dbName.recordQty	= 0;	\
							while( dbName.bufferPtr1 < dbName.eof )	\
							{	\
								while( dbName.buffer[dbName.bufferPtr1] != LF )	\
								{	\
									if( !dbName.buffer[dbName.bufferPtr1] )	\
									{	\
										dbName.fieldPtr1++;	\
									}	\
									dbName.bufferPtr1++;	\
								}	\
								if( dbName.fieldPtr1 != dbName.fieldQty )	\
								{	\
									fprintf( stderr,	\
				"MGmDbArray error: inconsitant field count at record %d. field count is	%d and should be %d\n",	\
										dbName.recordQty, dbName.fieldPtr1,	\
										dbName.fieldQty );	\
									errno = 1;	\
									break;	\
								}	\
								dbName.bufferPtr1++;	\
								dbName.recordQty++;	\
								dbName.fieldPtr1	= 0;	\
							}	\
							if( errno==0 )	\
							{	\
								if(!(dbName.record=(char**)malloc((unsigned int)dbName.recordQty*(unsigned int)dbName.fieldQty*(unsigned int)sizeof(char**))))	\
								{																					\
									fprintf( stderr,																\
										"MGmDbArray error: realloc failed to allocate %d bytes for records\n",		\
										dbName.fieldQty * sizeof( char* ) );	\
									perror( "system message" );	\
									errno = 1;	\
								}	\
								if( errno==0 )	\
								{	\
									dbName.recordStartAddress	= dbName.record;	\
									if( !( dbName.array = ( char*** )malloc((unsigned int)dbName.recordQty * (unsigned int)sizeof( char** ) ) ) )	\
									{	\
										fprintf( stderr,	\
											"MGmDbArray error: malloc failed to allocate %d bytes for dbNamease\n", 	\
											dbName.recordQty * sizeof( char** ) );	\
										perror( "system message" );	\
										errno = 1;	\
									}	\
									if( errno==0 )	\
									{	\
										dbName.field	= dbName.buffer;	\
										dbName.record[0]	= ( char* )dbName.field;	\
										dbName.array[0]	= ( char** )dbName.record;	\
										dbName.recordIdx		= 0;	\
										while( dbName.recordIdx < dbName.recordQty )	\
										{	\
											dbName.fieldIdx	= 0;	\
											while( 1==1 )	\
											{	\
												dbName.record[dbName.fieldIdx]	= dbName.field;	\
												dbName.fieldPtr1	= ( int )dbName.field;	\
												if( !( dbName.field = strchr( ( char* )dbName.fieldPtr1, 0) ) )	\
												{	\
													fprintf( stderr, "MGmDbArray error: next field not found\n" );	\
													perror( "system message" );	\
													errno = 1;	\
													break;	\
												}	\
												dbName.field = dbName.field + 1;	\
												dbName.fieldIdx++;	\
												if( dbName.field[0] == LF )	\
												{	\
													dbName.field = dbName.field + 1;	\
													break;	\
												}	\
											}	\
											if( errno )	\
											{	\
												break;	\
											}	\
											dbName.array[dbName.recordIdx]		= ( char** )dbName.record;	\
											dbName.fieldIdx	= 0;	\
											while( dbName.fieldIdx < dbName.fieldQty )	\
											{	\
												dbName.record++;	\
												dbName.fieldIdx++;	\
											}	\
											dbName.recordIdx++;	\
										}	\
									}	\
								}	\
							}	\
						}	\
					}	\
				}	\
			}	\
		}	\
	}}	\
	while (0)

#define MGmSetString( var, string )				\
	errorCode	= 0;					\
	if( !( var = ( char* )malloc((unsigned int)strlen( string ) + 1 ) ) )\
	{							\
		errorCode	= 1;				\
	}							\
	strncpy( var, string, strlen(string)+1 );

int			MGrFileSize( char* );
unsigned long int	MGrFileTime( char* );
int			MGrIfFileExist( char* );
int			MGrInStringSwap( char*, char*, char* );
char*			MGrIntToString( int );
int			MGrStripComment( char*, char );

#endif
